
# q-expansion of newform 3856.2.a.h, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3856
weight = 2
poly_data = [-13, -30, 26, 16, -10, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[-1, -6, 0, 1, 0, 0], 2], [[7, -4, -2, 1, 0, 0], 2], [[4, 9, -6, -2, 1, 0], 2], [[3, 17, 3, -9, -1, 1], 2]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0], [-1, 0, 0, 0, 0, 1], [-2, -1, 0, 1, 1, 0], [1, 0, 1, -1, 0, 0], [1, 0, 0, 1, 0, 0], [0, 0, -1, 1, 0, -1], [-1, 0, -1, -1, 0, 0], [-2, 0, -1, 1, 0, 1], [0, 2, 1, -1, -2, -1], [-3, 0, 1, 0, 2, 1], [3, -3, -1, 2, 3, 0], [0, -1, -1, -1, 0, 2], [-4, 2, -1, 1, 0, -1], [0, 4, 1, -3, 0, -1], [-1, 2, 2, 0, -2, -1], [0, -1, 0, 2, -2, -1], [-1, 0, -1, 3, 2, 0], [-1, 1, 3, -1, 0, -1], [0, -2, 0, -1, 2, 3], [3, 2, 2, -2, -2, -1], [-1, 2, -1, -1, -4, -2], [0, 3, 0, 2, -2, -2], [-3, -2, 1, -1, 0, 0], [-3, -3, 1, 3, 0, 2], [-6, 3, 2, -1, -1, -2], [-2, 4, 2, -4, 0, 0], [0, 1, 1, -3, -2, 2], [-4, -2, -3, -1, 2, 3], [-3, -1, 2, -4, -4, -1], [-1, -1, 1, -2, -1, 2], [2, 8, 2, -4, -2, 0], [-4, 2, -3, 1, -2, 1], [3, -4, -2, 0, 2, -1], [-6, -2, -2, 3, 2, -1], [-3, 2, -5, 3, 0, -4], [0, 6, 0, -3, 0, -1], [-5, 4, -5, 1, -2, -2], [-7, -4, 1, 4, 2, 1], [2, -1, 1, -2, 3, 1], [3, 4, -1, -3, -4, 2], [5, 2, -1, -1, 2, -4], [-13, -2, -1, 1, 0, 2], [-2, -1, -1, 3, -2, -1], [5, -4, -1, -2, 2, 1], [-4, 9, 5, -4, -3, -3], [-5, -1, 2, 6, 0, 1], [3, 2, 3, -7, -4, 0], [5, 0, 1, -7, -6, -4], [-5, 6, 1, -3, -2, -2], [5, 3, 3, -1, 2, -1], [2, -6, -3, 3, 2, 1], [1, 0, 0, 0, 0, 0], [-7, -3, -1, 3, 0, 4], [5, -3, -1, 3, -2, -2], [-1, 0, 3, -4, -2, -1], [-2, 2, 3, -3, -2, 3], [-7, 0, -1, 3, -6, -4], [13, -4, 0, 2, 0, -3], [-5, 3, 1, 1, -6, -2], [12, 4, 6, -4, -4, -2], [9, 0, 4, -8, 0, 5], [-3, -6, 0, 6, 6, -5], [-4, 3, -4, -3, -5, 4], [-3, -2, 3, -3, 6, 4], [11, 9, 0, -9, -1, 1], [-7, -11, 0, 6, 8, -2], [8, -3, -4, 2, 0, -8], [-4, -5, -2, -4, -2, 4], [-10, -10, -6, 8, 4, 2], [-3, 0, -5, 5, 0, -2], [-7, 4, 5, -3, 2, 6], [11, -6, -3, 1, -2, -2], [13, 0, 0, 3, 2, -2], [6, -4, -8, 2, 4, 0], [-3, -3, -6, 9, 1, -3], [-3, 0, 2, 5, 2, -8], [6, 6, 4, -10, -2, 0], [0, -11, -6, 4, 4, 7], [-7, 6, 2, -4, 0, -5], [5, -10, 0, 4, 2, 1], [-7, 4, -1, -3, -8, -4], [2, 3, 2, -5, -7, -6], [11, 1, -1, -3, 4, 2], [1, -4, -1, 2, 4, -1], [-7, -4, -4, 4, 8, 1], [-5, -8, 0, 0, 6, 1], [3, 3, 1, -5, 2, 7], [-2, 1, 7, -2, -1, -3], [11, -4, -2, -5, -4, 2], [0, -3, 3, -3, -6, 0], [1, 4, 2, 2, -6, -7], [-4, 2, -2, 8, 4, -8], [-13, 2, 0, 6, -2, -7], [16, 4, 3, -9, -2, 5], [-14, -3, 2, -3, -3, -4], [-3, 0, -3, -5, -6, 8], [5, -8, -4, 4, 0, 5], [5, -7, 2, -2, -4, -4], [9, -18, -8, 10, 4, 7], [-14, -6, -8, 10, 4, 2], [11, -8, -3, 3, -4, 0], [0, 8, -7, 1, 0, -3], [2, 3, 5, -1, 4, -1], [-6, 0, -2, 0, -2, 10], [15, -10, 2, 4, 6, -1], [0, -2, -8, 6, 8, 2], [8, 4, 1, -7, -6, 7], [-11, 0, -2, 5, 2, -6], [4, 3, -2, 4, 0, -3], [-3, 2, 5, 1, -2, 10], [-3, -9, 3, 4, 5, -2], [1, -3, -6, 6, 2, 5], [19, -8, -5, 1, -2, -2], [-5, 16, 1, 2, -2, -3], [5, -11, -6, 8, 0, -5], [10, 9, 0, -4, -8, -2], [3, -9, -5, 8, -3, -2], [14, -6, 1, 8, 6, 8], [-5, -3, 2, 4, 6, 7], [-13, 3, 0, -3, -5, 3], [8, 1, 0, -8, 0, 1], [-7, 0, -5, 2, 0, 5], [-13, 3, -1, -3, 6, 4], [13, 6, -1, -7, 2, -2], [0, -5, 5, -10, 1, 9], [-5, 7, -3, 2, 3, 4], [-17, -14, -2, 10, 8, -3], [18, 8, 2, -6, -2, 2], [3, 2, 2, -8, -6, 5], [-5, 1, 0, 2, 10, 9], [-12, 2, 12, 2, -6, 2], [-6, 6, 2, 7, 8, -3], [-7, 9, -5, 2, 5, 4], [-6, 8, 4, -6, 4, 4], [7, 6, 3, -7, -8, 2], [-12, 4, 2, 0, 2, -4], [22, -9, -4, 4, 6, 5], [-5, -12, -9, 8, 6, -3], [-7, -2, 11, 9, -4, 0], [2, 2, 8, 0, 4, 0], [-6, -6, -1, 5, 4, -11], [14, -14, -2, 0, -2, -2], [7, -4, -4, -4, -2, 7], [10, 8, 9, -9, -4, -7], [-14, 6, 1, -1, -8, -3], [7, 0, -3, -11, 0, 0], [4, 5, 0, -6, 4, -3], [22, 0, 9, -3, -2, -3], [-35, -1, 4, 1, -1, -1], [10, 20, 1, -5, -8, 1], [25, -5, 3, -3, -2, -5], [9, 3, -8, 10, 8, 0], [-7, 12, -8, -8, -2, 3], [9, -12, 4, 0, -2, -5], [9, 0, 5, 5, -2, -8], [-19, 4, 7, -1, -2, -4], [-14, 0, 9, -5, -2, 1], [12, 3, -7, -3, -8, -4], [-18, -3, 8, 11, 11, -2], [28, -8, -8, -2, -6, 0], [16, 8, -1, 7, 0, 5], [14, -12, -6, -2, 6, 4], [23, 6, 11, -5, -8, -12], [-3, 8, -6, 0, -4, -5], [-11, 0, -10, 15, 4, -4], [2, 4, -3, 5, 0, -1], [4, -2, 1, 2, -4, 4], [-6, 1, 5, -5, 4, 12], [-8, 2, -2, -2, -2, -10], [3, -6, 13, -1, 2, 4], [-9, 1, -8, 3, 1, -5], [-9, 8, 2, -6, 10, 7], [8, 0, -6, -2, 8, 2], [19, -6, -7, 9, -2, 0], [-19, 6, 0, 4, 2, -9], [8, -11, -3, 5, 4, 5], [-8, 16, 5, -7, -4, -7], [-4, -3, 0, 7, 5, 4], [-6, -1, -10, -1, 5, 4], [-7, 6, 7, 1, 2, -8], [22, -2, -3, 1, 4, 3], [-2, -18, -2, 4, 6, -6], [-8, -5, 2, -4, -4, 2], [5, -7, 10, 5, 3, 3], [-15, -10, -5, 11, 2, -10], [1, 13, 9, -12, -3, -6], [6, -12, -2, 10, 8, 12], [-11, -4, 0, 12, 0, -7], [-33, 6, -2, 2, 2, 1], [12, -16, 1, 9, 0, -3], [-28, -2, -4, 4, 8, 2], [-25, 0, 1, -1, -2, 6], [3, -16, -11, 7, 16, 6], [4, 1, -3, 7, -2, -6], [9, 13, -7, -7, -10, 3], [17, 13, -7, -9, -8, 2], [-17, -10, -1, 7, 0, 4], [-34, 3, -8, 2, 4, 5], [-16, -6, -4, 6, 10, 4], [0, -22, -9, 11, 0, 11], [-13, -3, 4, -5, -11, 1], [16, -6, -1, 11, 18, -1], [-4, -8, -1, 1, 14, 7], [-19, -17, 3, -3, 6, -6], [-23, -12, -9, 17, 8, -6], [11, -3, 1, -8, -7, -2], [-32, 0, -3, -7, 0, 3], [-25, 0, 1, -5, 2, 0], [-12, 4, 0, 16, 4, -4], [-8, -2, -5, 15, 12, 3], [-1, -2, 8, 10, -6, 5], [1, -12, -4, 4, 8, 5], [-3, 4, 1, -5, 0, -2], [-15, 5, 1, 0, 7, -2], [18, 1, -2, 8, 0, -4], [8, 0, 12, -2, 2, 4], [-9, 4, -10, 16, 6, 1], [-16, 12, 3, -9, 6, 7], [-23, 9, 3, -2, -5, -2], [8, 5, 1, -8, -7, 3], [3, 4, 1, -7, -6, 16], [5, -6, 13, -15, 0, 4], [-7, 0, 16, 3, 4, -4], [-14, 10, 0, 8, -8, -16], [-12, 2, -7, 4, -10, -10], [1, 0, 3, -5, 4, 16], [3, 4, -9, 0, -12, -1], [-4, 20, 15, -15, -10, 3], [12, 7, -1, 7, 6, -2], [-9, 24, 3, -5, -8, -2], [-35, 16, -2, -2, -4, -5], [-25, 12, 5, -11, 0, -2], [-2, 16, 0, 0, -4, -2], [-20, -6, 1, -5, 4, 5], [17, -8, -8, 6, 6, -3], [20, -8, -9, -1, 10, 3], [13, -10, 7, -5, -6, -4], [-1, -13, -11, 3, -4, 11], [-21, 3, 10, -13, -11, 1], [-10, 13, 12, -10, -22, -5], [18, -12, 5, -3, 12, 13], [0, 16, 6, -8, -10, 8], [23, 4, 8, -1, -2, 0], [-4, 1, -1, -1, -14, -11], [18, -6, -16, 0, 4, 0], [-9, 12, 2, -12, 8, 13], [-16, -20, -16, 12, 16, 12], [4, 19, 0, -16, -4, 0], [-3, -16, 8, -7, 2, 2], [4, 20, 4, -6, -10, 8], [-5, -10, -1, -1, 6, 10], [14, 2, 10, -6, -10, 2], [-8, 8, -1, 9, -10, -13], [4, -6, 13, -9, -10, -1], [7, 18, 8, -14, -12, -11], [4, 2, -11, 6, -2, 8], [-10, 19, -4, 2, -2, -7], [-33, 14, 5, -9, -12, -8], [4, 3, 6, 8, -8, -10], [-19, 16, -8, 8, 0, -7], [-23, -3, -4, 14, 4, 6], [-5, 4, 5, 1, 12, 8], [-2, -6, 5, -5, 6, 9], [-8, 19, 6, -14, -12, -15], [0, 12, -4, 8, 0, -4], [11, 16, -5, -17, -4, 2], [-13, -8, -14, 12, 14, 9], [6, -9, -6, -14, 2, 15], [7, -11, -2, -11, 9, 9], [11, -8, -7, 1, 10, 4], [-23, 25, 1, -15, -8, -10], [3, 4, 3, -11, -2, -12], [19, -8, -2, 14, 12, -3], [23, 1, 6, -10, 6, 6], [-19, 6, 9, 17, 2, -6], [16, -23, -2, 2, 10, 11], [33, -9, -3, 10, 1, 4], [2, -4, -1, -1, 22, 11], [35, -6, 11, 1, -2, 0], [18, 2, -2, -3, 6, 7], [-11, -10, -12, -2, 6, 3], [34, 6, -9, -5, 6, 1], [-7, -6, 17, -13, 4, 4], [22, -13, -6, 14, 14, 11], [-13, -20, -4, 13, -2, 8], [15, -4, 6, -12, -8, 3], [1, 21, -6, -7, -11, 3], [1, -2, -6, 10, 2, 1], [27, -20, -7, 7, -6, -2], [-18, -4, 1, 5, -6, -7], [-13, -4, 11, -1, 12, 6], [-11, 19, 6, 12, 2, -14], [17, 13, 1, 5, -8, -5], [13, 4, -7, 13, -6, -10], [30, -2, 1, 2, 6, -6], [13, 8, 0, -11, 4, 6], [-12, -6, 4, -6, 12, 6], [13, 14, 20, -18, -10, 3], [-33, 1, 3, 9, 4, 3], [-4, -18, -3, -1, 2, -5], [-35, 12, 5, -7, 8, 6], [-8, -16, 0, -6, -8, 4], [-21, -6, 5, 19, 2, 2], [4, 1, 3, -7, -6, -15], [32, -6, 1, 11, 6, 1], [32, -4, -3, 9, 20, -1], [-22, -6, 6, -17, -6, -1], [-20, 13, -3, -4, 3, -1], [-37, 6, -9, 7, 0, 2], [-11, 8, -5, 1, -2, -4], [-14, 6, 14, -4, 0, 0], [-10, 8, 8, -2, -10, -8], [-18, 8, 12, -12, 2, 8], [6, -8, 6, -4, 4, -6], [-21, 1, 3, -11, -12, 6], [-12, 2, -1, -9, 4, 1], [-28, -10, 1, 15, 12, -1], [-7, 4, 11, -11, 10, -2], [-13, -7, -4, 16, 18, 4], [10, 10, 4, -2, -6, -2], [8, 3, 1, -5, 8, -7], [24, 6, -8, 10, 2, -10], [7, 10, -10, 6, -6, -1], [21, 2, 5, -21, -10, -2], [-24, 23, -4, -8, 0, -1], [-10, -7, 5, 7, 2, -7], [-29, 14, -1, -11, -6, 0], [7, -16, 1, 6, 8, 9], [-7, -4, -10, -1, -10, 6], [24, -5, 2, -1, 13, 2], [-10, -14, 1, -12, 16, 12], [0, 14, 16, -13, -4, -9], [20, -10, -4, 4, 14, 4], [-15, -15, -12, -6, 12, 10], [36, -4, -4, 12, -4, 4], [-11, 18, -3, 1, -4, -12], [-8, 2, 4, 12, -4, -6], [9, 0, 6, 0, 2, -1], [28, 2, -17, 15, -8, -5], [-3, 16, 7, -16, -2, 11], [-14, -8, 1, -3, -4, 1], [-7, -16, 15, 0, 10, -1], [8, -6, -2, 5, 12, 9], [-5, -10, -19, 19, 18, 2], [-25, -8, 11, -9, 10, 6], [-27, 16, -8, -8, -8, 9], [-8, 6, -4, -10, -18, -2], [22, -12, 5, 7, -4, -11], [-8, -6, 19, 5, 10, -3], [-17, -2, -1, -11, 8, 22], [28, 12, -10, 4, -8, 0], [24, 20, 15, -5, -12, -9], [-18, -4, -3, -5, 10, 1], [-9, 16, 7, -13, 0, -10], [28, -24, -2, -14, -4, 10], [-43, 4, 12, -13, -16, 4], [8, -12, -12, 8, 0, -4], [-17, 20, 9, -5, -2, 14], [6, 5, 5, -16, -11, 11], [7, 28, 14, -6, -16, -15], [13, 2, 15, -9, -4, -6], [-5, -12, -11, 10, 0, -11], [31, 5, 3, -5, -4, 2], [-38, -4, -9, 21, 12, 1], [27, 2, 8, -10, -8, 1], [0, -14, -8, -2, 16, 4], [-24, 13, 6, 1, 3, -4], [7, -2, -7, 1, -16, -22], [-17, -3, -4, 20, 4, 3], [-15, 2, 2, 16, 8, -5], [-2, -6, -12, 10, -6, 4], [11, 4, 13, 1, -12, -4], [39, -10, -1, -3, 2, 2], [10, -12, 5, 15, 12, 7], [-30, 3, -5, 3, 12, 9], [-11, 22, 7, -3, 6, -2], [7, -11, 5, 9, 6, -3], [31, -4, -14, 14, -4, 1], [-1, 0, 7, 17, 0, -8], [-16, -2, -2, -14, -2, 2], [-13, -1, 1, -1, -2, 15], [-44, 4, 2, -10, -12, 6], [8, -4, 8, -4, -4, 6], [12, 1, -11, 11, -10, -2], [30, -8, -13, -5, -12, 7], [-25, -2, -12, 10, -8, -15], [-33, 4, 1, 9, 8, 4], [-31, 9, 7, -1, 12, 9], [22, -18, -10, 8, 12, 0], [-10, -10, -11, 9, 14, 19], [-5, 24, 6, 3, -2, -16], [20, -15, -3, 9, 6, -2], [-23, -14, -4, 28, 22, -5], [-8, 16, 5, -5, -12, -3], [26, 8, 6, -24, -4, 10], [6, -7, -2, 13, 3, -14], [-14, -6, 14, -8, -8, 10], [6, 17, -7, -9, -6, 8], [21, 6, 1, 3, 4, -12], [13, 12, 11, -17, -18, 0], [26, 6, 4, -20, -14, 0], [-32, -4, -7, 1, -6, 3], [-31, -18, -12, 28, 12, -3], [-14, 11, -9, -4, -7, -11], [-27, -4, -2, -3, -6, 2], [26, -2, 14, 1, 8, -11], [21, 9, -2, -10, 6, 1], [25, 12, -5, -1, -12, 4], [14, 8, -6, 12, -4, -10], [16, 28, 14, -22, -12, -8], [-11, -15, 6, 9, 3, -7], [10, -9, -1, -17, 2, 4], [6, -17, -12, 12, 16, 6], [14, -16, -10, 12, 6, -8], [11, -4, -15, 12, -6, 5], [-22, 1, 9, 4, -1, 5], [18, 6, 7, -9, -2, 13], [-17, -17, -10, 12, 12, -11], [4, -19, 8, -5, -7, -4], [-28, -5, -2, -5, -17, 8], [40, 4, 5, -13, -2, 7], [5, 9, 21, -20, -3, -12], [-3, 9, -13, -15, -10, 6], [-42, 17, -4, -2, -4, -13], [-4, -25, -7, 22, 3, -5], [17, -20, -3, 11, 10, -2], [-6, 0, 5, 1, 16, 5], [7, 1, -7, 17, 16, -12], [-17, 24, -8, 4, 0, 1]]
